; ---------------------------------------------------------------------------------------------------
; Acionamento de LEDs atraves do Smartphone via Bluetooth                                  18/11/2019
;
; STATUS: FUNCIONA
;
; Este programa recebe strings de um aplicativo do Smartphone Android desenvolvido no MIT App Inventor.
;
; O aplicativo para o Smartphone Android, que reside na pasta /Android/PicCMD.apk deste projeto deve 
; ser instalado no Android.
;
; O dispositivo Bluetooth usado e' o HC6 conectado ao Android usando a USART com
; Baud Rate de 9600BPS. Notar que o Baud Rate depende da frequencia do clock (Fosc) utilizada
; no projeto, neste caso 20MHZ. Consultar 'Baud Rate Generator' no data sheet do microcontrolador a
; respeito do registrador SPBRG.
;
; A leitura da mensagem sera feita atraves da interrupcao de Rx da porta serial (USART).
; A interrupcao lera uma mensagem completa, delimitada por um caracter de fim de string enviado pelo
; aplicativo do Smartphone. 
;
; O caracter de fim de string deve ser definido neste programa atraves do simbolo 'dlmchar'.
;
; A mensagem sera colocada no buffer de leitura 'hc6buffer' que pode ser alocado em qualquer banco da
; memoria RAM. 
;
; Neste programa, as mensagens consistem de um byte de comando e um byte delimitador de fim, a saber:
;                         hc6buffer = '1/' --> liga/desliga o LED 1
;                         hc6buffer = '2/' --> liga/desliga o LED 2  
;
; ---------------------------------------------------------------------------------------------------
; HARDWARE
; CLOCK ................... Cristal 20MHZ
; RB1/Rx .................. ao Tx do HC6 Bluetooth 
; RB2/Tx .................. ao Rx do HC6 Bluetooth (cuidado: 3,3v)
; RB4 ..................... LED1
; RB5 ..................... LED2
;
; ---------------------------------------------------------------------------------------------------
;

	LIST P=16F628A
	#include <P16F628A.INC>
    errorlevel -302
     __config  _MCLRE_ON & _LVP_OFF & _WDT_OFF & _PWRTE_OFF & _BODEN_OFF & _HS_OSC
;      	__CONFIG   _MCLRE_ON & _LVP_OFF & _WDT_OFF & _PWRTE_OFF & _BODEN_OFF & _INTOSC_OSC_NOCLKOUT 
; ---------------------------------------------------------------------------------------------
;

            UDATA         0x20 

dlmchar     equ         "/"       ; caracter delimitador de fim da string que se espera
                                  ; receber do aplicativo do Smartphone                                  

; LED1 - Definicao de I/O
#define     led1Tris    TRISB
#define     led1Mask    b'00010000'
#define     led1Port    PORTB
#define     led1Bit     .4

; LED2 - Definicao de I/O
#define     led2Tris    TRISB
#define     led2Mask    b'00100000'
#define     led2Port    PORTB
#define     led2Bit     .5

wreg        res       .1      ; salvamento dw W na interrupcao
sreg        res       .1      ; salvamento de STATUS na interrupcao
flags       res       .1      ; flags do programa
irx         equ       .0      ; 1 --> indica que ocorreu interrupcao da porta serial (Rx)

hc6buffer   equ     0x140     ; Buffer de leitura do HC6 Bluetooth definido no banco 2.
                              ; Ver data sheet.

;
; ---------------------------------------------------------------------------------------------
RESET_VECTOR CODE    0x000 
             goto    inicio
INT_VECTOR   CODE    0x004 
             goto    FLIH
;
; ---------------------------------------------------------------------------------------------
;
MAIN        CODE 

; ---------------------------------------------------------------------------------------------
; Configuracaoes iniciais
; ---------------------------------------------------------------------------------------------
inicio      movlw   0x07                 ; selecao de comparadores
            movwf   CMCON                ; desliga comparadores
            clrf    PORTA
            clrf    PORTB
            bsf     STATUS,RP0           ; banco 1
            movlw   0xFB                 ;  NC,   NC,  MCLR,  NC,   NC, BUT,  NC, NC
            movwf   TRISA                ;  1     1     1     1     1    0    1    1 
            movlw   0xCF                 ; PGD, PGC, LED, LED,   NC, TX, RX, NC
            movwf   TRISB                ;  1    1    0     0     1   1   1   1
            bcf     STATUS,RP0           ; banco 0
            clrf    flags                ; limpa os flags do programa

; -----------------------------------------------------------------------------------
; Configuracao da USART para operar o dispositivo Bluetooth.
; Configura interrupcao de Rx para receber caracteres do dispositivo Bluetooth.
; -----------------------------------------------------------------------------------
           bsf     STATUS, RP0            ; banco 1
           bsf     TRISB, 2               ; Tx
           bsf     TRISB, 1               ; Rx
        ;   movlw   .25                    ; 9600BPS a 4 MHZ --> 25 (ver data sheet)
           movlw   .129                   ; 9600BPS a 20MHZ --> 129 (ver data sheet)
           movwf   SPBRG                  ; SET BAUD RATE GENERATOR
           bsf     TXSTA,  BRGH           ; High Speed
           bcf     TXSTA,  SYNC           ; Async Mode 
           bcf     STATUS, RP0            ; banco 0
           bsf     RCSTA,  SPEN           ; Enable Serial Port
           bsf     RCSTA,  CREN           ; Enable Reception
           bsf     INTCON, PEIE           ; Enable Peripheric Interruptions
           bsf     INTCON, GIE            ; Enable General Interruptions

; -----------------------------------------------------------------------------------
; Rotina principal
; -----------------------------------------------------------------------------------

           ; Prepara para receber uma mensagem completa do Bluetooth
nextstep   bcf     flags, irx            ; limpa indicador de interrupcao Rx
           bcf     PIR1, RCIF            ; limpa flag da interrupcao
           bsf     STATUS, RP0           ; banco 1
           bsf     PIE1,   RCIE          ; habilita interrupcao Rx
           bcf     STATUS, RP0           ; banco 0   

           ; Espera chegar mensagem do Bluetooth   
loop       btfss   flags, irx            ; chegou mensagem do Bluetooth?
           goto    loop                  ; NAO --> espera chegar mensagem

           ; Desabilita a interrupcao Rx para processar a mensagem
verCmd     bsf     STATUS, RP0           ; banco 1
           bcf     PIE1,   RCIE          ; desabilita interrupcao Rx
           bcf     STATUS, RP0           ; banco 0   

           ; Processa a mensagem recem chegada
           movlw   hc6buffer             ; endereco do inicio do buffer
           movwf   FSR                   ; carrega indexador

           movlw   "1"                   ; SIM --> 
           xorwf   INDF, w               ; verifica caracter recebido
           btfsc   STATUS, Z             ; fim da mensagem?
           goto    led1Tog               ; SIM --> vai executar procedimento

           movlw   "2"                   ; SIM --> 
           xorwf   INDF, w               ; verifica caracter recebido
           btfsc   STATUS, Z             ; fim da mensagem?
           goto    led2Tog               ; SIM --> vai executar procedimento

           goto    nextstep              ; vai esperar nova mensagem do Bluetooth

           ; Inverte estado do LED 1
led1Tog    movlw   led1Mask
           xorwf   led1Port, f
           goto    nextstep

           ; Inverte estado do LED 2
led2Tog    movlw   led2Mask
           xorwf   led2Port, f
           goto    nextstep

; ---------------------------------------------------------------------------------------------
; INTERRUPCOES: Rotina de tratamento da interrupcao (Rx) da porta serial.
;
; A leitura da mensagem enviada pelo modulo Bluetooth HC6 sera feita atraves desta interrupcao.
;
; O buffer de leitura 'hc6buffer' deve ser grande o suficiente para conter a mensagem e,
; teoricamente, pode residir em qualquer banco da memoria RAM visto que o acesso a ele sera
; feito indiretamente atraves do registrador FSR.
;
; Entrada: 1 - O simbolo 'dlmchar' deve ser definido com o caracter delimitador de fim de string.
;          2 - O bit flags<irx> devera estar desligado.
;          3 - O nome do buffer tem que ser obrigatoriamente 'hc6buffer'.
;
; Saida  : 1 - O bit flags<irx> sera ligado indicando que ha nova mensagem no buffer.
;          2 - O buffer de leitura 'hc6buffer' sera preenchido com caracteres enviados pelo 
;              dispositivo Bluetooth ate que o caracter delimitador de fim da mensagem 'dlmchar'
;              seja recebido.
;
; ---------------------------------------------------------------------------------------------
FLIH       
           ; Salva o contexto
           movwf   wreg                ; salva W (MSB + LSB)
           swapf   STATUS,w            ; obtem o STATUS
           movwf   sreg                ; salva o STATUS antes da interrupca
           ; Enderecamento do buffer de leitura
           movlw   hc6buffer           ; inicio do buffer de leitura
           movwf   FSR                 ; carrega o indexador
           ; Pega o caracter lido na porta serial
vfychar    movf    RCREG, w            ; le o caracter enviado pelo Android
           movwf   INDF                ; armazena o caracter no endereco atual do buffer
           ; Verifica se houve erro de recepcao
           btfss   RCSTA, OERR         ; erro de recepcao???
           goto    norxerr             ; NAO --> segue normal
           bcf     RCSTA, CREN         ; SIM --> desliga recepcao para eliminar o erro
           nop                         ; delay para estabilizar
           nop                         ; delay para estabilizar
           nop                         ; delay para estabilizar
           bsf     RCSTA, CREN         ; religa a recepcao      
           ; Verifica se e' fim da string   
norxerr    xorlw   dlmchar             ; verifica fim da string
           btfsc   STATUS, Z           ; fim da string ?
           goto    eostr               ; SIM --> encerra interrupcao
           incf    FSR, f              ; avanca apontador do buffer
           bcf     PIR1, RCIF          ; limpa flag da interrupcao
           ; Le a porta serial
wnxchar    btfss   PIR1, RCIF          ; chegou o proximo caracter ?
           goto    wnxchar             ; NAI --> espera o proximo
           goto    vfychar             ; SIM --> vai verificar o caracter
           ; Restaura contexto
eostr      swapf   sreg,w              ; recupera STATUS
           movwf   STATUS              ; restaura o STATUS antes da interrupcao
           swapf   wreg,f              ; inverte nibles (LSB + MSB)
           swapf   wreg,w              ; restaura W sem afetar o STATUS
           bsf     flags, irx          ; indica ocorrencia de interrupcao Rx
           ; Retorna reabilitando a interrupcao
           bcf     PIR1, RCIF          ; limpa flag da interrupcao
           bsf     STATUS, RP0         ; banco 1
           bsf     PIE1,   RCIE        ; habilita interrupcao Rx
           bcf     STATUS, RP0         ; banco 0   
           retfie                      ; fim

           END
